@@ -24,9 +24,15 @@ DisplaySympy::DisplaySympy(const Kernel& kernel, const Ex& e)
 			{"\\cosh", "cosh"},
 			{"\\sinh", "sinh"},
 			{"\\tanh", "tanh"},
+         {"\\coth", "coth"},
+         {"\\sech", "sech"},
+         {"\\csch", "csch"},
 			{"\\arccosh", "acosh"},
 			{"\\arcsinh", "asinh"},
 			{"\\arctanh", "atanh"},
+         {"\\arccoth", "acoth"},
+         {"\\arcsech", "asech"},
+         {"\\arccsch", "acsch"},
 			{"\\log", "log"},
 			{"\\int", "integrate" },
 			{"\\matrix", "Matrix" },
